! TODO: event does not return if querydate == enddate

program test_mtime

  use mtime

  implicit none

  character(len=max_timedelta_str_len)   :: td_string
  character(len=max_datetime_str_len)    :: dt_string

  type(datetime),      pointer :: mtime_refdate       => null()
  type(datetime),      pointer :: mtime_startdate     => null()
  type(datetime),      pointer :: mtime_enddate       => null()
  type(timedelta),     pointer :: mtime_interval      => null()
                                                      
  type(event),         pointer :: mtime_event         => null()
  type(event_basic),   pointer :: mtime_event_basic   => null()
  type(event_generic), pointer :: mtime_event_generic => null()
  type(datetime),      pointer :: ev_startdate        => null()
  type(datetime),      pointer :: ev_enddate          => null()
  type(timedelta),     pointer :: ev_interval         => null()
  type(timedelta),     pointer :: ev_slack            => null()
  type(datetime),      pointer :: mtime_dt
  type(datetime),      pointer :: trigger_date1, trigger_date3
  integer                      :: ierr, i
  logical                      :: res1, res2, res3

  call setcalendar(proleptic_gregorian)

  ! offset for stop from reference date
  mtime_startdate  => newdatetime ("2016-05-29t00:00:00.000")
  mtime_enddate    => newdatetime ("2016-06-10t00:00:00.000")
  mtime_interval   => newtimedelta("pt59.900s")

  mtime_refdate    => newdatetime ("2016-05-29t00:00:00.000")
  ev_interval      => newtimedelta("pt10m")
  ev_slack         => newtimedelta("pt30s")
  ev_startdate     => newdatetime ("2016-05-29t00:10:00.000")
  ev_enddate       => newdatetime ("2018-06-01t00:00:00.000")

  trigger_date1  => newdatetime ("2010-01-01t00:00:00.000")
  trigger_date3  => newdatetime ("2010-01-01t00:00:00.000")

  call datetimetostring (mtime_refdate,    dt_string)
  write (0,*) "reference date    : ",      dt_string
  call datetimetostring (mtime_startdate,  td_string)
  write (0,*) "loop start time   : ",      td_string
  call datetimetostring (mtime_enddate,    td_string)
  write (0,*) "loop stop time    : ",      td_string
  call timedeltatostring (mtime_interval,  td_string)
  write (0,*) "model time step   : ",      td_string

  call datetimetostring (ev_startdate,     td_string)
  write (0,*) "event start time  : ",      td_string
  call datetimetostring (ev_enddate,       td_string)
  write (0,*) "event stop time   : ",      td_string
  call timedeltatostring (ev_interval,     td_string)
  write (0,*) "event interval    : ",      td_string


  ! QUESTIONS:
  !
  ! The "standard events" do not trigger at 2016-05-29T00:10:00.000 ?!

  write (0,*) ""

  mtime_event         => newevent('mec', mtime_refdate, ev_startdate, ev_enddate, ev_interval, errno=ierr)
  if (ierr /= 0) write (0,*) "standard event initialization: ERROR!"
  mtime_event_basic   => newbasicevent('mec', mtime_refdate, ev_startdate, ev_enddate, ev_interval, &
       &                               plus_slack=ev_slack, errno=ierr)
  if (ierr /= 0) write (0,*) "Basic event initialization: ERROR!"
  mtime_event_generic => newgenericevent('mec', mtime_refdate, ev_startdate, ev_enddate, ev_interval, &
       &                                 plus_slack=ev_slack, errno=ierr)
  if (ierr /= 0) write (0,*) "Generic event initialization: ERROR!"
  write (0,*) "associated(std) = ", associated(mtime_event_generic%ev_std)
  write (0,*) "associated(basic) = ", associated(mtime_event_generic%ev_basic)

  mtime_dt => newdatetime(mtime_startdate)
  i = 0
  do
     call datetimetostring(mtime_dt, dt_string)
     ! write (0,*) "    i = ", i, "current date: ", trim(dt_string)

     res1 = (iscurrenteventactive (mtime_event, mtime_dt, plus_slack=ev_slack))
     res2 = (iscurrentbasiceventactive (mtime_event_basic, mtime_dt))
     res3 = (iscurrentgenericeventactive (mtime_event_generic, mtime_dt))

     if (res1) then
        i = i + 1
        if (i<10)  write (0,*) "OLD: trigger at date-time stamp: ", trim (dt_string)
     end if
     if (res2) then
        if (i<10)  write (0,*) "NEW: trigger at date-time stamp: ", trim (dt_string)
     end if
     if (res1 .neqv. res2) then
        call datetimetostring (mtime_dt, dt_string)
        write (0,*) "date-time stamp: ", trim (dt_string)
        write (0,*) "results differ between event and basic event!!!"
     end if
     if (res1 .neqv. res3) then
        call datetimetostring (mtime_dt, dt_string)
        write (0,*) "date-time stamp: ", trim (dt_string)
        write (0,*) "results differ between event and generic event!!!"
     end if

     ! compare trigger dates
     if ((res1 .or. res3) .and. (i<=10)) then
        call getTriggeredPreviousEventAtDateTime(mtime_event,trigger_date1)
        res3 = (iscurrentgenericeventactive (mtime_event_generic, mtime_dt, trigger_date3))
        call datetimetostring(mtime_dt, dt_string)
        write (0,*) "    i = ", i, "current date: ", trim(dt_string)
        call datetimetostring (trigger_date1, dt_string)
        write (0,*) "trigger_date (standard): ", trim (dt_string)
        call datetimetostring (trigger_date3, dt_string)
        write (0,*) "trigger_date (generic):  ", trim (dt_string)
        write (0,*) " "
     end if

     if (mtime_dt >= mtime_enddate)  exit
     mtime_dt = mtime_dt + mtime_interval
  end do
  call deallocateDatetime( mtime_dt)

  write (0,*) "Done."

end program test_mtime
